;====================================================================
;File: remap_GASPsmoke_NAM.pro
;Purpose: remap GASP smoke into corresponding HYSPLIT NAM grid
;Author:  Jian Zeng
;Created: 2006-07-26
;
;Notes:  nlat--------------# of latitude for HYSPLIT GRID
;        nlon--------------# of longitude for HYSPLIT GRID
;        dlat--------------grid size of latitude for HYSPLIT GRID
;        dlon--------------grid size of longitude for HYSPLIT GRID
;        clat--------------latitude lower-left corner for HYSPLIT
;        clon--------------longitude lower-left corner for HYSPLIT
;INPUT:  goes_infile-------GASP data file
;        nx_g--------------# of elements for GASP data
;        ny_g--------------# of lines for GASP data
;        glat--------------GASP latitude
;        glon--------------GASP longitude file
;output:
;        outdata-----------combined aod and cloudy/clear
;                          -9999.0: no data
;                          -1: cloudy
;                          -2: no fires found
;                          -3: non-smoke aerosols
;                          -0.5~2.05: smoke AOD
;====================================================================
pro remap_GASPsmoke_NAM,nlat,nlon,dlat=dlat,dlon=dlon,clat=clat,$
                        clon=clon,glat=glat,glon=glon,$
			indata=indata,outdata=outdata,$
			num_smokeaod=num_smokeaod,$
			sum_smokeaod=sum_smokeaod,$
			num_cloud=num_cloud,num_nonsmoke=num_nonsmoke,$
			num_nofire=num_nofire,num_nodata=num_nodata

    nxny_file='/data/GASP/US/ref/USnxny'
    openr,1,nxny_file
      readf,1,outnx,outny
      nx_g = outnx
      ny_g = outny
    close,1

   print,'nx_g, ny_g'
   print,nx_g,ny_g

    ;set GASP default grid sizes
;    if n_elements(nx_g) eq 0 then nx_g=2000
;    if n_elements(ny_g) eq 0 then ny_g=850

    ;HYSPLIT GRID information
    ;get_hysplitgrid3,nlat,nlon,dlat,dlon,clat,clon

    outdata=fltarr(nlon,nlat)-9999.0
    
    if not keyword_set(num_smokeaod) then begin
      sum_smokeaod=fltarr(nlon,nlat)+0.
      num_smokeaod=intarr(nlon,nlat)+0L
      num_cloud=intarr(nlon,nlat)+0L
      num_nonsmoke=intarr(nlon,nlat)+0L
      num_nofire=intarr(nlon,nlat)+0L
      num_nodata=intarr(nlon,nlat)+0L

      ;remapping
      for i=0,nx_g-1 do begin
      for j=0,ny_g-1 do begin
        if glon(i,j) ge -180. and glat(i,j) ge -90. then begin

          ii=fix((glon(i,j)-clon+0.5*dlon)/dlon)
          jj=fix((glat(i,j)-clat+0.5*dlat)/dlat)
          if ii ge 0 and jj ge 0 and ii lt nlon and jj lt nlat then begin
	    if indata(i,j) lt -9998.0 then $
	      num_nodata(ii,jj)=num_nodata(ii,jj)+1L else $
	    if indata(i,j) eq -1. then $
	      num_cloud(ii,jj)=num_cloud(ii,jj)+1L else $
	    if indata(i,j) eq -2. then $
	      num_nofire(ii,jj)=num_nofire(ii,jj)+1L else $
	    if indata(i,j) eq -3. then $
	      num_nonsmoke(ii,jj)=num_nonsmoke(ii,jj)+1L else $
	    if indata(i,j) ge -0.5 and indata(i,j) le 2.05 then begin
	      sum_smokeaod(ii,jj)=sum_smokeaod(ii,jj)+indata(i,j)
              num_smokeaod(ii,jj)=num_smokeaod(ii,jj)+1L
            endif
          endif
        endif
      endfor
      endfor
    endif

    lable=[99.0,-1.,-3.,-2.,-9999.0]
    for i=0,nlon-1 do begin
      for j=0,nlat-1 do begin
        temp=[num_smokeaod(i,j),num_cloud(i,j),num_nonsmoke(i,j),$
	      num_nofire(i,j),num_nodata(i,j)]
	if max(temp) gt 0 then begin
	  result=max(temp,loc)
	  if loc eq 0L then begin
            outdata(i,j)=sum_smokeaod(i,j)/num_smokeaod(i,j)
	  endif else outdata(i,j)=lable(loc)

	endif

      endfor
    endfor


end

